@@ -14,18 +14,17 @@ use {
         genesis_utils::{create_genesis_config_with_leader_ex, GenesisConfigInfo},
     },
     solana_sdk::{
-        account::{Account, AccountSharedData, ReadableAccount},
+        account::{Account, AccountSharedData, ReadableAccount, WritableAccount},
         account_info::AccountInfo,
         clock::{Clock, Slot},
         entrypoint::{ProgramResult, SUCCESS},
         epoch_schedule::EpochSchedule,
-        feature_set::demote_program_write_locks,
+        feature_set::demote_sysvar_write_locks,
         fee_calculator::{FeeCalculator, FeeRateGovernor},
         genesis_config::{ClusterType, GenesisConfig},
         hash::Hash,
         instruction::Instruction,
         instruction::InstructionError,
-        keyed_account::KeyedAccount,
         message::Message,
         native_token::sol_to_lamports,
         process_instruction::{
@@ -99,16 +98,22 @@ fn get_invoke_context<'a>() -> &'a mut dyn InvokeContext {
 pub fn builtin_process_instruction(
     process_instruction: solana_sdk::entrypoint::ProcessInstruction,
     program_id: &Pubkey,
-    keyed_accounts: &[KeyedAccount],
     input: &[u8],
     invoke_context: &mut dyn InvokeContext,
 ) -> Result<(), InstructionError> {
     set_invoke_context(invoke_context);
 
+    let keyed_accounts = invoke_context.get_keyed_accounts()?;
+
     // Copy all the accounts into a HashMap to ensure there are no duplicates
-    let mut accounts: HashMap<Pubkey, AccountSharedData> = keyed_accounts
+    let mut accounts: HashMap<Pubkey, Account> = keyed_accounts
         .iter()
-        .map(|ka| (*ka.unsigned_key(), ka.account.borrow().clone()))
+        .map(|ka| {
+            (
+                *ka.unsigned_key(),
+                Account::from(ka.account.borrow().clone()),
+            )
+        })
         .collect();
 
     // Create shared references to each account's lamports/data/owner
@@ -153,7 +158,7 @@ pub fn builtin_process_instruction(
         let mut account = keyed_account.account.borrow_mut();
         let key = keyed_account.unsigned_key();
         let (lamports, data, _owner) = &account_refs[key];
-        account.lamports = **lamports.borrow();
+        account.set_lamports(**lamports.borrow());
         account.set_data(data.borrow().to_vec());
     }
 
@@ -167,13 +172,11 @@ macro_rules! processor {
     ($process_instruction:expr) => {
         Some(
             |program_id: &Pubkey,
-             keyed_accounts: &[solana_sdk::keyed_account::KeyedAccount],
              input: &[u8],
              invoke_context: &mut dyn solana_sdk::process_instruction::InvokeContext| {
                 $crate::builtin_process_instruction(
                     $process_instruction,
                     program_id,
-                    keyed_accounts,
                     input,
                     invoke_context,
                 )
@@ -256,14 +259,14 @@ impl solana_sdk::program_stubs::SyscallStubs for SyscallStubs {
             }
             panic!("Program id {} wasn't found in account_infos", program_id);
         };
-        let demote_program_write_locks =
-            invoke_context.is_feature_active(&demote_program_write_locks::id());
+        let demote_sysvar_write_locks =
+            invoke_context.is_feature_active(&demote_sysvar_write_locks::id());
         // TODO don't have the caller's keyed_accounts so can't validate writer or signer escalation or deescalation yet
         let caller_privileges = message
             .account_keys
             .iter()
             .enumerate()
-            .map(|(i, _)| message.is_writable(i, demote_program_write_locks))
+            .map(|(i, _)| message.is_writable(i, demote_sysvar_write_locks))
             .collect::<Vec<bool>>();
 
         stable_log::program_invoke(&logger, &program_id, invoke_context.invoke_depth());
@@ -334,25 +337,25 @@ impl solana_sdk::program_stubs::SyscallStubs for SyscallStubs {
 
         // Copy writeable account modifications back into the caller's AccountInfos
         for (i, account_pubkey) in message.account_keys.iter().enumerate() {
-            if !message.is_writable(i, demote_program_write_locks) {
+            if !message.is_writable(i, true) {
                 continue;
             }
 
             for account_info in account_infos {
                 if account_info.unsigned_key() == account_pubkey {
                     let account = &accounts[i];
-                    **account_info.try_borrow_mut_lamports().unwrap() = account.borrow().lamports;
+                    **account_info.try_borrow_mut_lamports().unwrap() = account.borrow().lamports();
 
                     let mut data = account_info.try_borrow_mut_data()?;
                     let account_borrow = account.borrow();
                     let new_data = account_borrow.data();
-                    if *account_info.owner != account.borrow().owner {
+                    if account_info.owner != account.borrow().owner() {
                         // TODO Figure out a better way to allow the System Program to set the account owner
                         #[allow(clippy::transmute_ptr_to_ptr)]
                         #[allow(mutable_transmutes)]
                         let account_info_mut =
                             unsafe { transmute::<&Pubkey, &mut Pubkey>(account_info.owner) };
-                        *account_info_mut = account.borrow().owner;
+                        *account_info_mut = *account.borrow().owner();
                     }
                     if data.len() != new_data.len() {
                         // TODO: Figure out how to allow the System Program to resize the account data
@@ -390,16 +393,6 @@ impl solana_sdk::program_stubs::SyscallStubs for SyscallStubs {
 }
 
 pub fn find_file(filename: &str) -> Option<PathBuf> {
-    for dir in default_shared_object_dirs() {
-        let candidate = dir.join(&filename);
-        if candidate.exists() {
-            return Some(candidate);
-        }
-    }
-    None
-}
-
-fn default_shared_object_dirs() -> Vec<PathBuf> {
     let mut search_path = vec![];
     if let Ok(bpf_out_dir) = std::env::var("BPF_OUT_DIR") {
         search_path.push(PathBuf::from(bpf_out_dir));
@@ -408,8 +401,15 @@ fn default_shared_object_dirs() -> Vec<PathBuf> {
     if let Ok(dir) = std::env::current_dir() {
         search_path.push(dir);
     }
-    trace!("BPF .so search path: {:?}", search_path);
-    search_path
+    trace!("search path: {:?}", search_path);
+
+    for path in search_path {
+        let candidate = path.join(&filename);
+        if candidate.exists() {
+            return Some(candidate);
+        }
+    }
+    None
 }
 
 pub fn read_file<P: AsRef<Path>>(path: P) -> Vec<u8> {
@@ -466,6 +466,7 @@ pub struct ProgramTest {
     builtins: Vec<Builtin>,
     bpf_compute_max_units: Option<u64>,
     prefer_bpf: bool,
+    use_bpf_jit: bool,
 }
 
 impl Default for ProgramTest {
@@ -495,18 +496,12 @@ impl Default for ProgramTest {
             builtins: vec![],
             bpf_compute_max_units: None,
             prefer_bpf,
+            use_bpf_jit: false,
         }
     }
 }
 
 impl ProgramTest {
-    /// Create a `ProgramTest`.
-    ///
-    /// This is a wrapper around [`default`] and [`add_program`]. See their documentation for more
-    /// details.
-    ///
-    /// [`default`]: #method.default
-    /// [`add_program`]: #method.add_program
     pub fn new(
         program_name: &str,
         program_id: Pubkey,
@@ -527,6 +522,11 @@ impl ProgramTest {
         self.bpf_compute_max_units = Some(bpf_compute_max_units);
     }
 
+    /// Execute the BPF program with JIT if true, interpreted if false
+    pub fn use_bpf_jit(&mut self, use_bpf_jit: bool) {
+        self.use_bpf_jit = use_bpf_jit;
+    }
+
     /// Add an account to the test environment
     pub fn add_account(&mut self, address: Pubkey, account: Account) {
         self.accounts
@@ -579,7 +579,7 @@ impl ProgramTest {
 
     /// Add a BPF program to the test environment.
     ///
-    /// `program_name` will also be used to locate the BPF shared object in the current or fixtures
+    /// `program_name` will also used to locate the BPF shared object in the current or fixtures
     /// directory.
     ///
     /// If `process_instruction` is provided, the natively built-program may be used instead of the
@@ -590,7 +590,20 @@ impl ProgramTest {
         program_id: Pubkey,
         process_instruction: Option<ProcessInstructionWithContext>,
     ) {
-        let add_bpf = |this: &mut ProgramTest, program_file: PathBuf| {
+        let loader = solana_sdk::bpf_loader::id();
+        let program_file = find_file(&format!("{}.so", program_name));
+
+        if process_instruction.is_none() && program_file.is_none() {
+            panic!("Unable to add program {} ({})", program_name, program_id);
+        }
+
+        if (program_file.is_some() && self.prefer_bpf) || process_instruction.is_none() {
+            let program_file = program_file.unwrap_or_else(|| {
+                panic!(
+                    "Program file data not available for {} ({})",
+                    program_name, program_id
+                );
+            });
             let data = read_file(&program_file);
             info!(
                 "\"{}\" BPF program from {}{}",
@@ -614,87 +627,28 @@ impl ProgramTest {
                     .unwrap_or_else(|| "".to_string())
             );
 
-            this.add_account(
+            self.add_account(
                 program_id,
                 Account {
                     lamports: Rent::default().minimum_balance(data.len()).min(1),
                     data,
-                    owner: solana_sdk::bpf_loader::id(),
+                    owner: loader,
                     executable: true,
                     rent_epoch: 0,
                 },
             );
-        };
-
-        let add_native = |this: &mut ProgramTest, process_fn: ProcessInstructionWithContext| {
+        } else {
             info!("\"{}\" program loaded as native code", program_name);
-            this.builtins
-                .push(Builtin::new(program_name, program_id, process_fn));
-        };
-
-        let warn_invalid_program_name = || {
-            let valid_program_names = default_shared_object_dirs()
-                .iter()
-                .filter_map(|dir| dir.read_dir().ok())
-                .flat_map(|read_dir| {
-                    read_dir.filter_map(|entry| {
-                        let path = entry.ok()?.path();
-                        if !path.is_file() {
-                            return None;
-                        }
-                        match path.extension()?.to_str()? {
-                            "so" => Some(path.file_stem()?.to_os_string()),
-                            _ => None,
-                        }
-                    })
-                })
-                .collect::<Vec<_>>();
-
-            if valid_program_names.is_empty() {
-                // This should be unreachable as `test-bpf` should guarantee at least one shared
-                // object exists somewhere.
-                warn!("No BPF shared objects found.");
-                return;
-            }
-
-            warn!(
-                "Possible bogus program name. Ensure the program name ({}) \
-                matches one of the following recognizable program names:",
+            self.builtins.push(Builtin::new(
                 program_name,
-            );
-            for name in valid_program_names {
-                warn!(" - {}", name.to_str().unwrap());
-            }
-        };
-
-        let program_file = find_file(&format!("{}.so", program_name));
-        match (self.prefer_bpf, program_file, process_instruction) {
-            // If BPF is preferred (i.e., `test-bpf` is invoked) and a BPF shared object exists,
-            // use that as the program data.
-            (true, Some(file), _) => add_bpf(self, file),
-
-            // If BPF is not required (i.e., we were invoked with `test`), use the provided
-            // processor function as is.
-            //
-            // TODO: figure out why tests hang if a processor panics when running native code.
-            (false, _, Some(process)) => add_native(self, process),
-
-            // Invalid: `test-bpf` invocation with no matching BPF shared object.
-            (true, None, _) => {
-                warn_invalid_program_name();
-                panic!(
-                    "Program file data not available for {} ({})",
-                    program_name, program_id
-                );
-            }
-
-            // Invalid: regular `test` invocation without a processor.
-            (false, _, None) => {
-                panic!(
-                    "Program processor not available for {} ({})",
-                    program_name, program_id
-                );
-            }
+                program_id,
+                process_instruction.unwrap_or_else(|| {
+                    panic!(
+                        "Program processor not available for {} ({})",
+                        program_name, program_id
+                    );
+                }),
+            ));
         }
     }
 
@@ -742,12 +696,19 @@ impl ProgramTest {
 
         let mut bank = Bank::new(&genesis_config);
 
-        for loader in &[
-            solana_bpf_loader_deprecated_program!(),
-            solana_bpf_loader_program!(),
-            solana_bpf_loader_upgradeable_program!(),
-        ] {
-            bank.add_builtin(&loader.0, loader.1, loader.2);
+        // Add loaders
+        macro_rules! add_builtin {
+            ($b:expr) => {
+                bank.add_builtin(&$b.0, $b.1, $b.2)
+            };
+        }
+        add_builtin!(solana_bpf_loader_deprecated_program!());
+        if self.use_bpf_jit {
+            add_builtin!(solana_bpf_loader_program_with_jit!());
+            add_builtin!(solana_bpf_loader_upgradeable_program_with_jit!());
+        } else {
+            add_builtin!(solana_bpf_loader_program!());
+            add_builtin!(solana_bpf_loader_upgradeable_program!());
         }
 
         // Add commonly-used SPL programs as a convenience to the user
